#include "riders.h"
#include <bits/stdc++.h>
#include <random>
using namespace std;

#ifdef EVAL
FILE* input = fopen("input.txt", "r");
FILE* output = fopen("output.txt", "w");
#else
istream& in(cin);
ostream& out(cout);
#endif

#define NONE -1
#define GREEDY_WEIGHTS 0
#define GREEDY_PROFITS_MINUS 1
#define GREEDY_PROFITS_DIVIDE 2
#define WEIGHTS_TIME 1500
#define PROFIT_MINUS_TIME 800
#define PROFIT_DIVIDE_TIME 800
#define RAND_INTERVALS 2
#define MAX_DEPTH 5

double best_solution = 0;
double start;
int N, K, S;
int curr_id = 0;
int sols_found = 0;
int heuristic, run_time;
bool using_paths = false;
typedef pair<int, int> ii;

std::random_device myRandomDevice;
unsigned seed = 69240;
std::default_random_engine myRandomEngine(seed);

int get_rand_int(int min, int max) {
    std::uniform_int_distribution<int> myUnifIntDist(min, max);
    return myUnifIntDist(myRandomEngine);
}

vector<int> owner;
vector<int> profit;
int graph[2000][2000];
int sorted_graph[2000][2000];
int best_sol = 0;
int INIT_PROB = 102;
int PROB;
int inter = 0;
int intra = 0;
int iter = 0;
bool sorted_by_profit = false;

struct Runner {
    int id;
    int t;
    int next_t;
    int current;
    int score;
    vector<int> path;

    Runner() {
        id = curr_id++;
        t = 0;
        next_t = 0;
        score = 0;
        current = S;
        path.push_back(S);
    }

    void add_edge(int to) {
        path.push_back(to);
        owner[to] = id;
        t += graph[current][to];
        current = to;
    }
};

vector<Runner> run;
vector<Runner> best_runners;
struct runner_cmp {
    bool operator()(int a, int b) {
        return run[a].next_t > run[b].next_t;
    }
};
priority_queue<int, vector<int>, runner_cmp> runner_q;

void sort_by_profit_minus() {
    for (int i = 0; i < N; i++) {
        sort(begin(sorted_graph[i]), begin(sorted_graph[i]) + N, [&](int i1, int i2) {
            return profit[i1] - graph[i][i1] > profit[i2] - graph[i][i2];
        });
    }
}

void sort_by_profit_divide() {
    for (int i = 0; i < N; i++) {
        sort(begin(sorted_graph[i]), begin(sorted_graph[i]) + N, [&](int i1, int i2) {
            return 1LL * profit[i1] * graph[i][i2] > 1LL * profit[i2] * graph[i][i1];
        });
    }
}

void sort_by_time() {
    for (int i = 0; i < N; i++) {
        sort(begin(sorted_graph[i]), begin(sorted_graph[i]) + N, [&](int i1, int i2) {
            return graph[i][i1] < graph[i][i2];
        });
    }
}

int compute_score() {
    int score = 0;
    for (int i = 0; i < K; i++) {
        score += run[i].score;
    }
    return score;
}

void print_sol() {
    best_runners = run;
    int score = compute_score();
    best_sol = score;
#ifdef EVAL
    fprintf(output, "%d\n", score);
#else
    out << score << "\n";
#endif
    for (int i = 0; i < K; i++) {
#ifdef EVAL
        fprintf(output, "%lu\n", best_runners[i].path.size());
#else
        out << best_runners[i].path.size() << "\n";
#endif
        for (int p : best_runners[i].path) {
#ifdef EVAL
            fprintf(output, "%d ", p);
#else
            out << p << " ";
#endif
        }
#ifdef EVAL
        fprintf(output, "\n");
#else
        out << "\n";
#endif
    }
#ifdef EVAL
    fprintf(output, "***\n");
#else
    out << "***"
        << "\n";
#endif
}

ii compute_path_score(vector<int>& path, int time = 0) {
    int score = 0;
    int last = NONE;
    for (int i = 0; i < path.size() - 1; i++) {
        int from = path[i];
        int to = path[i + 1];
        time += graph[from][to];
        if (profit[to] > time) {
            score += profit[to] - time;
            last = to;
        }
    }
    return {score, last};
}

void clean_path(vector<int>& path, int last) {
    while (path.back() != last) {
        path.pop_back();
    }
}

void intra_two_opt(int iter) {
    intra++;
    for (int i = 0; i < K; i++) {
        for (int t = 0; t < iter * run[i].path.size(); t++) {
            int u = get_rand_int(0, run[i].path.size() - 1);
            int v = get_rand_int(0, run[i].path.size() - 1);
            if (u != v && run[i].path[u] != S && run[i].path[v] != S) {
                swap(run[i].path[u], run[i].path[v]);
                ii new_score = compute_path_score(run[i].path);
                if (new_score.first > run[i].score) {
                    // cerr << "Found good swap! From " << run[i].score << " to " << new_score.first << endl;
                    run[i].score = new_score.first;
                    clean_path(run[i].path, new_score.second);
                } else {
                    swap(run[i].path[u], run[i].path[v]);
                }
            }
        }
    }
}

void inter_two_opt(int iter) {
    if (K == 1)
        return;
    inter++;
    for (int t = 0; t < iter; t++) {
        int r1 = get_rand_int(0, K - 1);
        int r2 = get_rand_int(0, K - 1);
        int u = get_rand_int(0, run[r1].path.size() - 1);
        int v = get_rand_int(0, run[r2].path.size() - 1);
        if (r1 != r2 && run[r1].path[u] != S && run[r2].path[v] != S) {
            swap(run[r1].path[u], run[r2].path[v]);
            ii path1 = compute_path_score(run[r1].path);
            ii path2 = compute_path_score(run[r2].path);
            int last1 = path1.second, last2 = path2.second;
            if (last1 != NONE && last2 != NONE && path1.first + path2.first > run[r1].score + run[r2].score) {
                run[r1].score = path1.first;
                clean_path(run[r1].path, last1);
                run[r2].score = path2.first;
                clean_path(run[r2].path, last2);
            } else {
                swap(run[r1].path[u], run[r2].path[v]);
            }
        }
    }
}

void prepare_go() {
    for (int i = 0; i < K; i++) {
        int time = 0;
        for (int p = 0; p < run[i].path.size() - 1; p++) {
            owner[run[i].path[p]] = run[i].id;
            time += graph[run[i].path[p]][run[i].path[p + 1]];
        }
        run[i].t = time;
        run[i].current = run[i].path.back();
        owner[run[i].path.back()] = run[i].id;
        runner_q.push(i);
    }
    owner[S] = N + 1;
}

int get_next_edge(int i) {
    int edge = NONE;
    int current = run[i].current;
    for (int j = 0; j < N; j++) {
        int to = sorted_graph[current][j];
        if (owner[to] == NONE && to != current && profit[to] > run[i].t + graph[current][to]) {
            if (get_rand_int(0, 100) < PROB) {
                edge = to;
                break;
            }
        }
    }
    return edge;
}

int compute_path_score_with_negatives(vector<int>& path, int time) {
    int score = 0;
    for (int i = 0; i < path.size() - 1; i++) {
        int from = path[i];
        int to = path[i + 1];
        time += graph[from][to];
        score += max(0, profit[to] - time);
    }
    return score;
}

int get_path_time(vector<int>& path) {
    int time = 0;
    for (int i = 0; i < path.size() - 1; i++) {
        int from = path[i];
        int to = path[i + 1];
        time += graph[from][to];
    }
    return time;
}

void get_next_path(int i, int depth, vector<int>& path, vector<int>& best_path, int& best_score) {
    if (depth == 0) {
        int path_score;
        if (heuristic == GREEDY_PROFITS_MINUS || heuristic == GREEDY_PROFITS_DIVIDE) {
            path_score = compute_path_score_with_negatives(path, run[i].t);
        } else {
            path_score = -get_path_time(path);
        }
        if (path_score > best_score && get_rand_int(0, 100) < PROB) {
            best_score = path_score;
            best_path = path;
        }
        return;
    }
    int current = path.back();
    int t = run[i].t + get_path_time(path);
    int paths = 1 + get_rand_int(0, 1);
    for (int j = 0; j < N; j++) {
        int to = sorted_graph[current][j];
        if (owner[to] == NONE && to != current && profit[to] > t + graph[current][to]) {
            // if(owner[to] == NONE && to != current) {
            owner[to] = i;
            path.push_back(to);
            get_next_path(i, depth - 1, path, best_path, best_score);
            path.pop_back();
            owner[to] = NONE;
            if (--paths == 0)
                break;
        }
    }
}

int get_edge(int i) {
    int edge = NONE;
    if (!using_paths) {
        edge = get_next_edge(i);
    } else {
        vector<int> path = {run[i].current};
        vector<int> best_path = path;
        int depth = 1 + ((clock() / 1000) - start) / (run_time / MAX_DEPTH);
        int best_score = INT_MIN;
        get_next_path(i, depth, path, best_path, best_score);
        if (best_path.size() > 1)
            edge = best_path[1];
    }
    return edge;
}

void setup() {
    vector<bool> assigned(N, false);
    owner.assign(N, NONE);
    owner[S] = N + 1;
    run.clear();
    run.resize(K);
    for (int i = 0; i < K; i++) {
        for (int j = 0; j < N; j++) {
            int edge = sorted_graph[S][j];
            if (edge != S && !assigned[edge] && profit[edge] > graph[S][edge]) {
                run[i].next_t = graph[S][edge];
                assigned[edge] = true;
                break;
            }
        }
        runner_q.push(i);
    }
}
int go() {
    int score = 0;
    PROB = INIT_PROB - ((clock() / 1000) - start) / (run_time / RAND_INTERVALS);
    // cerr << PROB << endl;
    while (!runner_q.empty()) {
        int i = runner_q.top();
        runner_q.pop();
        int edge = get_edge(i);
        if (edge != NONE) {
            run[i].add_edge(edge);
            run[i].score += max(0, profit[edge] - run[i].t);
            score += max(0, profit[edge] - run[i].t);
            int current = run[i].current;
            for (int j = 0; j < N; j++) {
                int next = sorted_graph[current][j];
                if (owner[next] == NONE) {
                    run[i].next_t = run[i].t + graph[current][next];
                    break;
                }
            }
            runner_q.push(i);
        }
    }
    return score;
}

void opt() {
    int a = 0;
    while (1) {
        int times = 10000;
        inter_two_opt(times);
        intra_two_opt(times);
        prepare_go();
        go();
        int score = compute_score();
        if (score > best_sol) {
            // cerr << "improved in opt by " << score - best_sol << endl;
            print_sol();
#ifdef EVAL
            fflush(output);
#else
            out.flush();
#endif
        }
    }
}

void go_heur() {
    while (clock() / 1000 - start < run_time) {
        setup();
        int score = go();
        if (score > best_sol) {
            print_sol();
        }
    }
}

void solve() {
    for (int i = 0; i < 1; i++) {
        start = clock() / 1000;
        sort_by_time();
        heuristic = GREEDY_WEIGHTS;
        run_time = WEIGHTS_TIME;
        go_heur();

        start = clock() / 1000;
        heuristic = GREEDY_PROFITS_MINUS;
        run_time = PROFIT_MINUS_TIME;
        sort_by_profit_minus();
        go_heur();

        start = clock() / 1000;
        heuristic = GREEDY_PROFITS_DIVIDE;
        run_time = PROFIT_DIVIDE_TIME;
        sort_by_profit_divide();
        go_heur();
        // using_paths = true;
    }
#ifdef EVAL
    fflush(output);
#else
    out.flush();
#endif
    run = best_runners;
    sort_by_time();
    heuristic = GREEDY_WEIGHTS;
    run_time = WEIGHTS_TIME;
    using_paths = 0;
    opt();
}

int main() {
    double a = clock() / 1000;
    ios_base::sync_with_stdio(false);
    cin.tie(NULL);
#ifdef EVAL
    assert(3 == fscanf(input, "%d %d %d", &N, &K, &S));
#else
    in >> N >> K >> S;
#endif

    profit.resize(N);
    for (int i = 0; i < N; i++) {
        iota(begin(sorted_graph[i]), begin(sorted_graph[i]) + N, 0);
    }

    for (int i = 0; i < N; i++) {
#ifdef EVAL
        assert(1 == fscanf(input, "%d", &profit[i]));
#else
        in >> profit[i];
#endif
    }

    int edge_weight;
    for (int i = 1; i < N; i++) {
        for (int j = 0; j < i; j++) {
#ifdef EVAL
            assert(1 == fscanf(input, "%d", &edge_weight));
#else
            in >> edge_weight;
#endif
            graph[i][j] = edge_weight;
            graph[j][i] = edge_weight;
        }
    }

    solve();
    double b = clock() / 1000;
    cerr << "took " << b - a << endl;
    return 0;
}
