#include <stdio.h>
#include <bits/stdc++.h>
#include <algorithm>
#include "riders.h"
using namespace std;

#ifdef EVAL
ifstream in("input.txt");
ofstream out("output.txt");
#else
istream &in(cin);
ostream &out(cout);
#endif

int N, K, S;
int t = 0;
int tot_profit = 0;

vector<vector<int>> graph;
vector<bool> visited;
vector<int> server_tgt;
vector<int> server_count;
vector<int> payoffs;
vector<vector<int>> servers_paths;
priority_queue<pair<int, int>> pq;


void read_input() {
    in >> N >> K >> S;
    graph.resize(N);
    visited.resize(N);
    payoffs.resize(N);
    server_count.resize(K);
    server_tgt.resize(K);
    servers_paths.resize(K);

    for(int i=0; i<N; i++) {
        graph[i].resize(N);
        visited[i] = false;
    }
    // source is visited
    visited[S] = true;

    for(int k=0; k<K; k++) {
        server_count[k] = -1;
        server_tgt[k] = -1;
        servers_paths[k].push_back(S);
    }

    int p;
    for(int i=0; i<N; i++) {
        in >> p;
        payoffs[i] = p;

        // ignore the payoff of the source
        if(i!=S) {
            pq.push(make_pair(p, i));
        }
    }

    int w;
    for(int i=1; i<N; i++) {
        for(int j=0; j<i; j++) {
            in >> w;
            graph[i][j] = w;
            graph[j][i] = w;
        }
    }

    return;
}


void print_input() {
    printf("N: %d, K: %d, S: %d\n", N, K, S);

    printf("-- payoffs:\n");
    for(int p : payoffs) {
        printf("%d ", p);
    }
    printf("\n");

    printf("-- graph:\n");
    for(int i=1; i<N; i++) {
        for(int j=0; j<i; j++) {
            printf("%*d ", 4, graph[i][j]);
        }
        printf("\n");
    }

    return;
}


void write_solution() {
    out << tot_profit << endl;
    for(int k=0; k<K; k++){
        out << servers_paths[k].size() << endl;
        for(int n: servers_paths[k]) {
            out << n << " ";
        }
        out << endl;
    }
    out << "***" << endl;

    return;
}

int main() {
    read_input();
    print_input();

    bool run = true;
    while(run) {
        printf("- time: %d\n", t);
        if(t==0) {
            // initial targets
            for(int k=0; k<K; k++) {
                if(!pq.empty()) {
                    pair<int, int> top = pq.top(); 
                    pq.pop();

                    int tgt = top.second;
                    int pos = S;
                    int dst = graph[S][tgt];

                    server_tgt[k] = tgt;
                    server_count[k] = dst;

                    printf("--> Initial target for server %d: %d! - ", k, tgt);
                    printf("dst(%d, %d): %d\n", pos, tgt, dst);
                }
            }
        }

        for(int k=0; k<K; k++) {
            if(server_count[k]==0) {
                // server is now at target
                int tgt = server_tgt[k];
                visited[tgt] = true;
                servers_paths[k].push_back(tgt);
                printf("--> Server %d reached target %d.\n", k, tgt);

                // collect profit, ensure that it is at least 0
                int profit = (payoffs[tgt] - t);
                tot_profit += max(0, profit);
                printf("--> Server %d collected profit %d.\n", k, profit);

                // get new target
                if(!pq.empty()) {
                    pair<int, int> top = pq.top(); 
                    pq.pop();

                    tgt = top.second;
                    int pos = servers_paths[k].back();
                    int dst = graph[pos][tgt];

                    printf("--> New target for server %d: %d! - ", k, tgt);
                    printf("dst(%d, %d): %d\n", pos, tgt, dst);

                    server_tgt[k] = tgt;
                    server_count[k] = dst;
                }
            }
        }


        if(all_of(visited.begin(), visited.end(), [](bool v) { return v; })) {
            run = false;
        }

        // tick time
        t++;
        for(int k=0; k<K; k++) {
            server_count[k]--;
        }
    }

    write_solution();

    return 0;
}
