#include <stdio.h>
#include <bits/stdc++.h>
#include <algorithm>
#include "riders.h"
using namespace std;

#ifdef EVAL
ifstream in("input.txt");
ofstream out("output.txt");
#else
istream &in(cin);
ostream &out(cout);
#endif

typedef pair<int, int> pint;
typedef priority_queue<pint,vector<pint>,greater<pint>> min_priority_queue;
int N, K, S;
int t = 0;
int tot_profit = 0;

vector<vector<int>> graph;
vector<bool> visited;
vector<bool> assigned;
vector<int> server_tgt;
vector<int> server_count;
vector<int> payoffs;
vector<vector<int>> servers_paths;
vector<min_priority_queue> minedges;


void read_input() {
    in >> N >> K >> S;
    graph.resize(N);
    visited.resize(N);
    assigned.resize(N);
    payoffs.resize(N);
    server_count.resize(K);
    server_tgt.resize(K);
    servers_paths.resize(K);

    for(int i=0; i<N; i++) {
        graph[i].resize(N);
        visited[i] = false;
    }
    // source is visited
    visited[S] = true;
    assigned[S] = true;

    for(int k=0; k<K; k++) {
        server_count[k] = -1;
        server_tgt[k] = -1;
        servers_paths[k].push_back(S);
    }

    int p;
    for(int i=0; i<N; i++) {
        in >> p;
        payoffs[i] = p;
    }

    int w;
    for(int i=1; i<N; i++) {
        for(int j=0; j<i; j++) {
            in >> w;
            graph[i][j] = w;
            graph[j][i] = w;
        }
    }

    minedges.resize(N);
    for(int i=0; i<N; i++) {
        for(int j=0; j<N; j++) {
            if(i!=j) {
                minedges[i].push(make_pair(graph[i][j], j));
            }
        }
    }

    return;
}


void print_input() {
    printf("N: %d, K: %d, S: %d\n", N, K, S);

    printf("-- payoffs:\n");
    for(int p : payoffs) {
        printf("%d ", p);
    }
    printf("\n");

    printf("-- graph:\n");
    for(int i=1; i<N; i++) {
        for(int j=0; j<i; j++) {
            printf("%*d ", 4, graph[i][j]);
        }
        printf("\n");
    }

    return;
}


void write_solution() {
    out << tot_profit << endl;
    for(int k=0; k<K; k++){
        out << servers_paths[k].size() << endl;
        for(int n: servers_paths[k]) {
            out << n << " ";
        }
        out << endl;
    }
    out << "***" << endl;

    return;
}

int main() {
    read_input();
    print_input();

    bool run = true;
    while(run) {
        printf("- time: %d\n", t);
        if(t==0) {
            // initial targets
            for(int k=0; k<K; k++) {
                if(!minedges[S].empty()) {
                    pair<int, int> top = minedges[S].top(); 
                    minedges[S].pop();

                    int tgt = top.second;
                    int pos = S;
                    int dst = graph[S][tgt];

                    assigned[tgt] = true;
                    server_tgt[k] = tgt;
                    server_count[k] = dst;

                    printf("--> Initial target for server %d: %d! - ", k, tgt);
                    printf("dst(%d, %d): %d\n", pos, tgt, dst);
                }
            }
        }

        for(int k=0; k<K; k++) {
            if(server_count[k]==0) {
                // server is now at target
                int tgt = server_tgt[k];
                visited[tgt] = true;
                servers_paths[k].push_back(tgt);
                printf("--> Server %d reached target %d.\n", k, tgt);

                // collect profit, ensure that it is at least 0
                int profit = (payoffs[tgt] - t);
                tot_profit += max(0, profit);
                printf("--> Server %d collected profit %d.\n", k, profit);

                // get new target
                int pos = servers_paths[k].back();
                int old_tgt = tgt;

                min_priority_queue tmp = minedges[pos];
                printf("--- pq[%d]: ", pos);
                while(!tmp.empty()) {
                    pair<int, int> top = tmp.top();
                    tmp.pop();
                    printf("(%d, %d) ", top.first, top.second);
                }
                printf("\n");
                printf("--- visited: ");
                for(auto v: visited) {
                    printf("%d ", v?1:0);
                }
                printf("\n");
                printf("--- assigned: ");
                for(auto a: assigned) {
                    printf("%d ", a?1:0);
                }
                printf("\n");

                while(tgt==old_tgt) {
                    if(!minedges[pos].empty()) {
                        pair<int, int> top = minedges[pos].top(); 
                        minedges[pos].pop();

                        if(!assigned[top.second]) {
                            tgt = top.second;
                            int dst = graph[pos][tgt];

                            printf("--> New target for server %d: %d! - ", k, tgt);
                            printf("dst(%d, %d): %d\n", pos, tgt, dst);

                            assigned[tgt] = true;
                            server_tgt[k] = tgt;
                            server_count[k] = dst;
                        }
                    } else {
                        break;
                    }
                }
            }
        }

        if(all_of(visited.begin(), visited.end(), [](bool v) { return v; })) {
            run = false;
        }

        // tick time
        t++;
        for(int k=0; k<K; k++) {
            server_count[k]--;
        }
    }

    write_solution();

    return 0;
}
